/*  gcdindex -- CDindex client
 *  Copyright (C) 1999 Nick Lamb <njl195@zepler.org.uk>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include "gladesig.h"
#include "gladesrc.h"

GtkWidget*
get_widget                             (GtkWidget       *widget,
                                        gchar           *widget_name)
{
  GtkWidget *parent, *found_widget;

  for (;;)
    {
      if (GTK_IS_MENU (widget))
        parent = gtk_menu_get_attach_widget (GTK_MENU (widget));
      else
        parent = widget->parent;
      if (parent == NULL)
        break;
      widget = parent;
    }

  found_widget = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (widget),
                                                   widget_name);
  if (!found_widget)
    g_warning ("Widget not found: %s", widget_name);
  return found_widget;
}

/* This is an internally used function to set notebook tab widgets. */
void
set_notebook_tab                       (GtkWidget       *notebook,
                                        gint             page_num,
                                        GtkWidget       *widget)
{
  GtkNotebookPage *page;
  GtkWidget *notebook_page;

  page = (GtkNotebookPage*) g_list_nth (GTK_NOTEBOOK (notebook)->children, page_num)->data;
  notebook_page = page->child;
  gtk_widget_ref (notebook_page);
  gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page_num);
  gtk_notebook_insert_page (GTK_NOTEBOOK (notebook), notebook_page,
                            widget, page_num);
  gtk_widget_unref (notebook_page);
}

static GList *pixmaps_directories = NULL;

/* Use this function to set the directory containing installed pixmaps. */
void
add_pixmap_directory                   (gchar           *directory)
{
  pixmaps_directories = g_list_prepend (pixmaps_directories, g_strdup (directory));
}

/* This is an internally used function to check if a pixmap file exists. */
#ifndef G_DIR_SEPARATOR_S
#define G_DIR_SEPARATOR_S "/"
#endif
gchar*
check_file_exists                      (gchar           *directory,
                                        gchar           *filename)
{
  gchar *full_filename;
  struct stat s;
  gint status;

  full_filename = g_malloc (strlen (directory) + 1 + strlen (filename) + 1);
  strcpy (full_filename, directory);
  strcat (full_filename, G_DIR_SEPARATOR_S);
  strcat (full_filename, filename);

  status = stat (full_filename, &s);
  if (status == 0 && S_ISREG (s.st_mode))
    return full_filename;
  g_free (full_filename);
  return NULL;
}

/* This is an internally used function to create pixmaps. */
GtkWidget*
create_pixmap                          (GtkWidget       *widget,
                                        gchar           *filename)
{
  gchar *found_filename = NULL;
  GdkColormap *colormap;
  GdkPixmap *gdkpixmap;
  GdkBitmap *mask;
  GtkWidget *pixmap;
  GList *elem;

  /* We first try any pixmaps directories set by the application. */
  elem = pixmaps_directories;
  while (elem)
    {
      found_filename = check_file_exists ((gchar*)elem->data, filename);
      if (found_filename)
        break;
      elem = elem->next;
    }

  /* If we haven't found the pixmap, try the source directory. */
  if (!found_filename)
    {
      found_filename = check_file_exists ("../pixmaps", filename);
    }

  if (!found_filename)
    {
      g_print ("Couldn't find pixmap file: %s", filename);
      return NULL;
    }

  colormap = gtk_widget_get_colormap (widget);
  gdkpixmap = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask,
                                                   NULL, found_filename);
  g_free (found_filename);
  if (gdkpixmap == NULL)
    return NULL;
  pixmap = gtk_pixmap_new (gdkpixmap, mask);
  gdk_pixmap_unref (gdkpixmap);
  gdk_bitmap_unref (mask);
  return pixmap;
}

GtkWidget*
create_main_window ()
{
  GtkWidget *main_window;
  GtkWidget *vbox1;
  GtkWidget *frame1;
  GtkWidget *table1;
  GtkWidget *title_label;
  GtkWidget *title_entry;
  GtkWidget *artist_label;
  GtkWidget *artist_entry;
  GtkWidget *frame2;
  GtkWidget *track_list;
  GtkWidget *hbuttonbox1;
  GtkWidget *refresh_button;
  GtkWidget *submit_button;
  GtkWidget *close_button;
  GtkWidget *statusbar;
  GtkTooltips *tooltips;

  tooltips = gtk_tooltips_new ();

  main_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (main_window), "main_window", main_window);
  gtk_signal_connect (GTK_OBJECT (main_window), "destroy",
                      GTK_SIGNAL_FUNC (finish),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (main_window), "delete_event",
                      GTK_SIGNAL_FUNC (gtk_widget_destroy),
                      NULL);
  gtk_window_set_title (GTK_WINDOW (main_window), "CDindex");
  gtk_window_set_policy (GTK_WINDOW (main_window), TRUE, TRUE, FALSE);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (main_window), "vbox1", vbox1);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (main_window), vbox1);

  frame1 = gtk_frame_new ("Disc");
  gtk_object_set_data (GTK_OBJECT (main_window), "frame1", frame1);
  gtk_widget_show (frame1);
  gtk_box_pack_start (GTK_BOX (vbox1), frame1, FALSE, TRUE, 0);
  gtk_container_border_width (GTK_CONTAINER (frame1), 3);

  table1 = gtk_table_new (2, 2, FALSE);
  gtk_object_set_data (GTK_OBJECT (main_window), "table1", table1);
  gtk_widget_show (table1);
  gtk_container_add (GTK_CONTAINER (frame1), table1);
  gtk_table_set_row_spacings (GTK_TABLE (table1), 10);
  gtk_table_set_col_spacings (GTK_TABLE (table1), 10);

  title_label = gtk_label_new ("Title");
  gtk_object_set_data (GTK_OBJECT (main_window), "title_label", title_label);
  gtk_widget_show (title_label);
  gtk_table_attach (GTK_TABLE (table1), title_label, 0, 1, 0, 1,
                    (GtkAttachOptions) 0, (GtkAttachOptions) 0, 0, 0);

  title_entry = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (main_window), "title_entry", title_entry);
  gtk_widget_show (title_entry);
  gtk_table_attach (GTK_TABLE (table1), title_entry, 1, 2, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) 0, 0, 0);
  GTK_WIDGET_UNSET_FLAGS (title_entry, GTK_CAN_FOCUS);
  gtk_entry_set_editable (GTK_ENTRY (title_entry), FALSE);
  gtk_entry_set_text (GTK_ENTRY (title_entry), "Full title of disc");

  artist_label = gtk_label_new ("Artist");
  gtk_object_set_data (GTK_OBJECT (main_window), "artist_label", artist_label);
  gtk_widget_show (artist_label);
  gtk_table_attach (GTK_TABLE (table1), artist_label, 0, 1, 1, 2,
                    (GtkAttachOptions) 0, (GtkAttachOptions) 0, 0, 0);

  artist_entry = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (main_window), "artist_entry", artist_entry);
  gtk_widget_show (artist_entry);
  gtk_table_attach (GTK_TABLE (table1), artist_entry, 1, 2, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) 0, 0, 0);
  GTK_WIDGET_UNSET_FLAGS (artist_entry, GTK_CAN_FOCUS);
  gtk_entry_set_editable (GTK_ENTRY (artist_entry), FALSE);
  gtk_entry_set_text (GTK_ENTRY (artist_entry), "Full name of artist");

  frame2 = gtk_frame_new ("Tracks");
  gtk_object_set_data (GTK_OBJECT (main_window), "frame2", frame2);
  gtk_widget_show (frame2);
  gtk_box_pack_start (GTK_BOX (vbox1), frame2, TRUE, TRUE, 0);
  gtk_container_border_width (GTK_CONTAINER (frame2), 3);

  track_list = gtk_list_new ();
  gtk_object_set_data (GTK_OBJECT (main_window), "track_list", track_list);
  gtk_widget_show (track_list);
  gtk_container_add (GTK_CONTAINER (frame2), track_list);
  gtk_widget_set_usize (track_list, 100, -2);

  hbuttonbox1 = gtk_hbutton_box_new ();
  gtk_object_set_data (GTK_OBJECT (main_window), "hbuttonbox1", hbuttonbox1);
  gtk_widget_show (hbuttonbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbuttonbox1, FALSE, TRUE, 0);
  gtk_container_border_width (GTK_CONTAINER (hbuttonbox1), 3);

  refresh_button = gtk_button_new_with_label ("Refresh");
  gtk_object_set_data (GTK_OBJECT (main_window), "refresh_button", refresh_button);
  gtk_widget_show (refresh_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), refresh_button);
  GTK_WIDGET_SET_FLAGS (refresh_button, GTK_CAN_DEFAULT);
  gtk_tooltips_set_tip (tooltips, refresh_button, "Fetch information from CDindex", NULL);
  gtk_signal_connect (GTK_OBJECT (refresh_button), "clicked",
                      GTK_SIGNAL_FUNC (refresh_clicked),
                      NULL);

  submit_button = gtk_button_new_with_label ("Submit");
  gtk_object_set_data (GTK_OBJECT (main_window), "submit_button", submit_button);
  gtk_widget_show (submit_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), submit_button);
  gtk_widget_set_sensitive (submit_button, FALSE);
  gtk_tooltips_set_tip (tooltips, submit_button, "Add this CD to CDindex", NULL);
  gtk_signal_connect (GTK_OBJECT (submit_button), "clicked",
                      GTK_SIGNAL_FUNC (submit_clicked),
                      NULL);

  close_button = gtk_button_new_with_label ("Close");
  gtk_object_set_data (GTK_OBJECT (main_window), "close_button", close_button);
  gtk_widget_show (close_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), close_button);
  GTK_WIDGET_SET_FLAGS (close_button, GTK_CAN_DEFAULT);
  gtk_signal_connect_object (GTK_OBJECT (close_button), "clicked",
                             GTK_SIGNAL_FUNC (gtk_widget_destroy),
                             GTK_OBJECT (main_window));

  statusbar = gtk_statusbar_new ();
  gtk_object_set_data (GTK_OBJECT (main_window), "statusbar", statusbar);
  gtk_widget_show (statusbar);
  gtk_box_pack_start (GTK_BOX (vbox1), statusbar, FALSE, TRUE, 0);

  gtk_object_set_data (GTK_OBJECT (main_window), "tooltips", tooltips);

  return main_window;
}

